#!/bin/bash

BIN_CHMOD="/bin/chmod"
BIN_MKDIR="/bin/mkdir -p"
BIN_LN="/bin/ln -sf"
BIN_RM="/bin/rm -rf"

DEFAULT_DIR_NAME=synoldap
DB_PATH=/usr/syno/etc/private
VOLUME_LDAP_DB_DIR="@${DEFAULT_DIR_NAME}"
ROOT_P_LDAP_DB_DIR="${DB_PATH}/@${DEFAULT_DIR_NAME}"
LDAP_DB_DIR_LINK="${DB_PATH}/${DEFAULT_DIR_NAME}"

logger_err()
{
	logger -s -p user.warn -t "ldap client" "$@"
}

create_dir() {
	local path=$1

	if [ ! -d "${path}" ]; then
		$BIN_RM "$path"
		$BIN_MKDIR "$path"
	fi
	$BIN_CHMOD 755 "$path"
}

init_env() {
	create_dir $ROOT_P_LDAP_DB_DIR
	$BIN_RM "${ROOT_P_LDAP_DB_DIR}/*.building"

	if [ ! -e $LDAP_DB_DIR_LINK ]; then
		$BIN_LN $ROOT_P_LDAP_DB_DIR $LDAP_DB_DIR_LINK
	fi
}

get_current_path() {
	local current
	current=$(/bin/realpath $LDAP_DB_DIR_LINK)
	if [ ! -d "$current" ]; then
		$BIN_RM $LDAP_DB_DIR_LINK
		$BIN_LN $ROOT_P_LDAP_DB_DIR $LDAP_DB_DIR_LINK
		current=$ROOT_P_LDAP_DB_DIR
	fi
	echo $current
}

get_curr_vol_path() {
	local -r vol_name="$(echo "${CURRENT_LDAP_DB_DIR}" | awk -F'/' '{print $2}')"
	echo "/${vol_name}"
}

is_vol_avail_size_enough () {
	local -r vol_avail_size="$(df -BM "${1}" | tail -1 | awk '{print $4}' | awk -F'M' '{print $1}')"
	if [ "${vol_avail_size}" -ge "10" ]; then
		return 0
	fi
	return 1
}

is_path_valid_volume() {
	local -r vol_path="$(get_curr_vol_path)"

	# check volume path valid
	if [ ! -e "${vol_path}" ] || [ "x${vol_path:0:7}" != "x/volume" ] \
		|| ! /bin/grep -q "${vol_path}" "/proc/mounts"; then
		return 1
	fi

	# check volume size is enough
	if is_vol_avail_size_enough "${vol_path}"; then
		return 0
	fi
	return 1
}

try_new_path() {
	local -r curr_vol_path="$(get_curr_vol_path)"
	if [ $# -ge 1 ] && [ "x${1}" != "x${curr_vol_path}" ]; then
		return
	fi

	$BIN_RM $LDAP_DB_DIR_LINK
	local vol_path
	vol_path="$(/usr/syno/bin/servicetool --get-alive-volume 10 "$1")"
	if [ ! -e "$vol_path" ] || [ "x${vol_path:0:7}" != "x/volume" ] || ! /bin/grep -q "${vol_path}" "/proc/mounts"; then
		logger_err "Can't find valid volume for db cache, use root partition."

		$BIN_LN $ROOT_P_LDAP_DB_DIR $LDAP_DB_DIR_LINK
		return
	fi

    logger_err "Found valid volume for ldap client db. [${vol_path}]"
	local new_path="${vol_path}/${VOLUME_LDAP_DB_DIR}"
	$BIN_RM "$new_path"
	create_dir "$new_path"
	$BIN_LN "$new_path" $LDAP_DB_DIR_LINK
}

init_env
CURRENT_LDAP_DB_DIR=$(get_current_path)

case "$1" in
	volume-unmount)
		try_new_path "$2"
		exit 0
		;;
	*)
		if ! is_path_valid_volume; then
			try_new_path
		fi
		exit 0
		;;
esac
