#!/bin/sh

PKG_NAME="SynoLog Volume Create Hook"
PKG_VERSION="1.0"
PKG_VENDOR="Synology Inc."
SDK_MODVER="1.0"

isSatadomModel() {
	if [ "`/bin/get_key_value /etc.defaults/synoinfo.conf root_device_synopartition`" = "no" ]; then
		return 0
	else
		return 1
	fi
}

case $1 in
	--sdk-mod-ver)
		echo ${SDK_MODVER}
	;;
	--name)
		echo ${PKG_NAME}
	;;
	--pkg-ver)
		echo ${PKG_VERSION}
	;;
	--vendor)
		echo ${PKG_VENDOR}
	;;
	--pre)
	;;
	--post)
		SYNOLOG_FOLDER="/var/log/synolog"
		VOLUME_SYNOLOG_FOLDER="$MOUNTPOINT/@database/synolog"
		SYNOLOG_DBS=".AFPXFERDB .DSMFMXFERDB .FTPXFERDB .SMBXFERDB .TFTPXFERDB .WEBDAVXFERDB"

		[ "$RESULT" -eq 0 ] || exit

		## check synolog path link exists and is a working link
		if isSatadomModel; then
			SYNOLOG_DBS="${SYNOLOG_DBS} .SYNOACCOUNTDB .SYNOCONNDB .SYNONETBKPDB"
			if [ -L "$SYNOLOG_FOLDER/.SYNOACCOUNTDB" ] && [ -f "$SYNOLOG_FOLDER/.SYNOACCOUNTDB" ]; then
				exit
			fi
		else
			if [ -L "$SYNOLOG_FOLDER/.SMBXFERDB" ] && [ -f "$SYNOLOG_FOLDER/.SMBXFERDB" ]; then
				exit
			fi
		fi

		[ -d "$VOLUME_SYNOLOG_FOLDER" ] || mkdir -p "$VOLUME_SYNOLOG_FOLDER"

		for db in $SYNOLOG_DBS; do
			if [ ! -L "$SYNOLOG_FOLDER/$db" ] && [ -f "$SYNOLOG_FOLDER/$db" ]; then
				/bin/mv "$SYNOLOG_FOLDER/$db" "$VOLUME_SYNOLOG_FOLDER/$db"
			fi
			/bin/ln -sf "$VOLUME_SYNOLOG_FOLDER/$db" "$SYNOLOG_FOLDER/$db"
		done

		/bin/chown -R system:log "$VOLUME_SYNOLOG_FOLDER"
	;;
	*)
		echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
	;;
esac
