#!/bin/sh

PKG_NAME="Synolog Volume Delete Hook"
PKG_VERSION="1.0"
PKG_VENDOR="Synology Inc."
SDK_MODVER="1.0"

isSatadomModel() {
	if [ "`/bin/get_key_value /etc.defaults/synoinfo.conf root_device_synopartition`" = "no" ]; then
		return 0
	else
		return 1
	fi
}

case $1 in
	--sdk-mod-ver)
		echo ${SDK_MODVER}
	;;
	--name)
		echo ${PKG_NAME}
	;;
	--pkg-ver)
		echo ${PKG_VERSION}
	;;
	--vendor)
		echo ${PKG_VENDOR}
	;;
	--pre)
		SYNOLOG_FOLDER="/var/log/synolog"
		SYNOLOG_DBS=".AFPXFERDB .DSMFMXFERDB .FTPXFERDB .SMBXFERDB .TFTPXFERDB .WEBDAVXFERDB"

		if isSatadomModel; then
			SYNOLOG_DBS="${SYNOLOG_DBS} .SYNOACCOUNTDB .SYNOCONNDB .SYNONETBKPDB"
			if [ "$(df "$SYNOLOG_FOLDER/.SYNOACCOUNTDB" | tail -1 | awk '{print $6}')" != "$MOUNTPOINT" ]; then
				exit
			fi
		else
			if [ "$(df "$SYNOLOG_FOLDER/.SMBXFERDB" | tail -1 | awk '{print $6}')" != "$MOUNTPOINT" ]; then
				exit
			fi
		fi

		SYNOLOG_DBSIZE=$(du -d0 -m "$MOUNTPOINT/@database/synolog" | awk '{print $1}')
		ALIVE_VOLUME="$(/usr/syno/bin/servicetool --get-alive-volume $SYNOLOG_DBSIZE $MOUNTPOINT)"
		OLD_VOLUME_SYNOLOG_FOLDER="$MOUNTPOINT/@database/synolog"
		NEW_VOLUME_SYNOLOG_FOLDER="$ALIVE_VOLUME/@database/synolog"

		if [ "${ALIVE_VOLUME:0:7}" = "/volume" ]; then
			# move synolog to another enough space volume
			logger -p 3 "Move synolog: $OLD_VOLUME_SYNOLOG_FOLDER -> $NEW_VOLUME_SYNOLOG_FOLDER."
			[ -d "$NEW_VOLUME_SYNOLOG_FOLDER" ] || mkdir -p "$NEW_VOLUME_SYNOLOG_FOLDER"
			for db in $SYNOLOG_DBS; do
				mv "$OLD_VOLUME_SYNOLOG_FOLDER/$db" "$NEW_VOLUME_SYNOLOG_FOLDER/$db"
				ln -sf "$NEW_VOLUME_SYNOLOG_FOLDER/$db" "$SYNOLOG_FOLDER/$db"
			done
			chown -R system:log "$NEW_VOLUME_SYNOLOG_FOLDER"
		else
			# no enough space, compress old synolog to any alive volume
			ALIVE_VOLUME="$(/usr/syno/bin/servicetool --get-alive-volume 0 $MOUNTPOINT)"
			NEW_VOLUME_SYNOLOG_FOLDER="$ALIVE_VOLUME/@database/synolog"

			if [ "${ALIVE_VOLUME:0:7}" = "/volume" ]; then
				logger -p 3 "No enough volume. Compress synolog to $NEW_VOLUME_SYNOLOG_FOLDER/synolog.txz."
				[ -d "$NEW_VOLUME_SYNOLOG_FOLDER" ] || mkdir -p "$NEW_VOLUME_SYNOLOG_FOLDER"
				/bin/tar cf "$NEW_VOLUME_SYNOLOG_FOLDER/synolog.txz" "$OLD_VOLUME_SYNOLOG_FOLDER"
				for db in $SYNOLOG_DBS; do
					ln -sf "$NEW_VOLUME_SYNOLOG_FOLDER/$db" "$SYNOLOG_FOLDER/$db"
				done
			else
				logger -p 3 "No alive volume. Drop all synolog ..."
				for db in $SYNOLOG_DBS; do
					unlink "$SYNOLOG_FOLDER/$db"
				done
			fi
		fi
	;;
	--post)
	;;
	*)
		echo "Usage: $0 --sdk-mod-ver|--name|--pkg-ver|--vendor|--pre|--post"
	;;
esac
