ControllerIP="169.254.1.1"
MACAddr=`ifconfig eth0 | grep "HWaddr" | cut -d"W" -f 2 | cut -d" " -f 2`
MntDir="/grinst"
NFSPath="/volume1/public"
ErrInstall="/tmp/installer.error"
GetKeyValue="/bin/get_key_value"
ProgressDir="${MntDir}/progress"
ProgressFile="${ProgressDir}/${MACAddr}"
GRINST_LOG="/tmp/grinst.$$"
Unique=`${GetKeyValue} /etc.defaults/synoinfo.conf unique` || true
Model=`${GetKeyValue} /etc.defaults/synoinfo.conf upnpmodelname` || true

# $1:   static blink
# green    8     9
# orange   :     ;
# Turn off if otherwise.
LedControl() {
	case $1 in
		"8"|"9"|":"|";") LED="$1" ;;
		*) LED="7" ;;
	esac
	echo "$LED" > /dev/ttyS1 || true

	if [ "${SupportLCM}" = "yes" ]; then
		echo "$LED" > /dev/ttyACM0 || true
	fi
}

LOG() {
	echo "$1"
	echo "[`date +%T`] $1" >> ${GRINST_LOG}
}

SetProgress() {
	LOG "=== $1"
	echo "[${Unique}] $1" > ${ProgressFile} || true
}

ErrorExit() {
	LOG "ERROR! $1"
	LedControl ":"
	exit 1
}

# $1/$2: Strings to compare, report $3 if not equal
# $3: Message to Report
ReportIfNotEqual() {
	if [ "$1" != "$2" ]; then
		SetProgress "$3"
		umount -f ${MntDir} || true
		ErrorExit
	fi
}

MountController() {
	if [ -n "$1" ]; then
		ControllerIP=$1
	fi

	mkdir -p ${MntDir} || true
	umount -f ${MntDir} || true
	if mount ${ControllerIP}:${NFSPath} ${MntDir}; then
		LOG "mount success" || true
		return 0
	else
		LOG "ERROR! Cannot mount ${ControllerIP}:${NFSPath} to ${MntDir}" || true
		return 1
	fi
}

UnMountController() {
	umount -f ${MntDir} || true
}

RunMemtester() {
	local burnin_data="$1"
	cp ${burnin_data}/memory_test /root/
	echo 3 > /proc/sys/vm/drop_caches
	sh /root/memory_test
	ReportIfNotEqual $? "0" "Failed to do memory_test"
}

GroupInstallSetup() {
	local burnin_data="$1"
	local script="group_install_setup.sh"
	if [ -f ${burnin_data}/${script} ]; then
		cp ${burnin_data}/${script} /root/
		sh /root/${script}
		ReportIfNotEqual $? "0" "Failed to setup group install"
	fi

	if [ -f ${burnin_data}/burnin_loader.sh ]; then
		SetProgress "${burnin_version} found burnin_loader.sh"
		cp -f ${burnin_data}/burnin_loader.sh /tmp/burnin_loader.sh
	fi
}

RunRemoteStarter() {
	local SupportWebinstall=`${GetKeyValue} /etc.defaults/synoinfo.conf support_webinstall`
	if [ "xyes" != "x${SupportWebinstall}" ]; then
		LOG "SupportWebinstall=${SupportWebinstall} is invalid"
		return
	fi
	if [ "synology_88f6282_213air" == "${Unique}" ]; then
		LOG "213air without curl in junior mode"
		return
	fi
	local ConfFullBurnin="${MntDir}/fullburnin.conf"
	if [ ! -f ${ConfFullBurnin} ]; then
		LOG "${ConfFullBurnin} not found"
		return
	fi
	local BurninStarterScript=`${GetKeyValue} ${ConfFullBurnin} remote_starter_script`
	if [ "x" == "x${BurninStarterScript}" ]; then
		LOG "remote_starter_script is empty in ${ConfFullBurnin}"
		return
	fi
	local BurninUsername=`${GetKeyValue} ${ConfFullBurnin} remote_username`
	if [ "x" == "x${BurninUsername}" ]; then
		LOG "remote_username is empty in ${ConfFullBurnin}"
		return
	fi
	local BurninPasswd=`${GetKeyValue} ${ConfFullBurnin} remote_passwd`
	local CookieFile="/tmp/burnin_server_cookie"
	local ApiOpt=""
	local ret=1

	curl -k \
		-d "username=${BurninUsername}&passwd=${BurninPasswd}" \
		"https://${ControllerIP}:5001/webman/login.cgi" \
		-D ${CookieFile}
	ret=$?
	if [ "0" != "${ret}" ]; then
		curl -k \
			-d "username=${BurninUsername}&passwd=${BurninPasswd}" \
			"http://${ControllerIP}:5000/webman/login.cgi" \
			-D ${CookieFile}
		ret=$?
	fi
	ReportIfNotEqual "${ret}" "0" "Failed to login burnin server"
	SetProgress "Login burnin server successfully..."

	ApiOpt="${ApiOpt}api=SYNO.Burnin.Server.Starter"
	ApiOpt="${ApiOpt}&method=download"
	ApiOpt="${ApiOpt}&version=1"

	curl -k -b ${CookieFile} -G \
		--data-urlencode "model=${Model}" \
		--data-urlencode "unique=${Unique}" \
		"https://${ControllerIP}:5001/webapi/entry.cgi?${ApiOpt}" \
		-o /tmp/starter.sh
	ret=$?
	if [ "0" != "${ret}" ]; then
		curl -k -b ${CookieFile} -G \
			--data-urlencode "model=${Model}" \
			--data-urlencode "unique=${Unique}" \
			"http://${ControllerIP}:5000/webapi/entry.cgi?${ApiOpt}" \
			-o /tmp/starter.sh
		ret=$?
	fi
	ReportIfNotEqual "$ret" "0" "Failed to download starter.sh"
	SetProgress "Download starter.sh successfully, execute starter.sh ..."
	sh /tmp/starter.sh
	ReportIfNotEqual $? "0" "Failed to run starter.sh"
	LOG "Finish starter.sh"
	umount -f ${MntDir} || true
	exit 0
}
