#!/usr/bin/python
# Copyright (c) 2000-2020 Synology Inc. All rights reserved.

from utils import *
from datetime import *


class RuleDictResult(DictResult):
    _non_comp_version = [DEFAULT_NON_COMP_VERSION]
    _group = [GROUP_HOME, GROUP_COMPANY]
    _category = CATEGORY_USERINFO
    _severity = LEVEL_RISK
    _strId = 'rule_disable_admin_dsm6'

    def getStatus(self):
        self._result = SZ_ERROR

        resp = execWebAPI("SYNO.Core.User", "get", 1, name="admin", additional=["expired"])
        if not resp or not resp["success"] or not resp["data"]["users"]:
            return self._result

        try:
            expired = resp["data"]["users"][0]["expired"]
            if "now" == expired:
                self._result = SZ_PASS
            elif "normal" == expired:
                self._result = SZ_FAIL
            elif (datetime.strptime(expired, "%Y/%m/%d") + timedelta(days=1)) <= datetime.today():
                self._result = SZ_PASS
            else:
                self._result = SZ_FAIL
        except Exception as e:
            SYSLOG(syslog.LOG_ERR, "Fail to check admin status: {}".format(e))
            return SZ_ERROR

        return self._result

    def getMethod(self):
        return {
            METHOD_ACTION: METHOD_ACTION_LINK,
            METHOD_ACTION_VAL: "SYNO.SDS.AdminCenter.Application:SYNO.SDS.AdminCenter.User.Main"
        }

    def getAction(self):
        if SZ_PASS != self._result:
            return {
                ACTION_STR_KEY: "action",
                ACTION_REPLACE_VAR: {
                    "%0": "_T('tree', 'leaf_user')"
                }
            }
        else:
            return {}
